from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.validator_problem_result import ValidatorProblemResult


T = TypeVar("T", bound="ProblemProgressEntry")


@_attrs_define
class ProblemProgressEntry:
    """Progress entry for a single problem showing results from all validators.

    Attributes:
        problem_id (UUID): Problem ID
        category (None | str | Unset): Problem category
        validator_results (list[ValidatorProblemResult] | Unset): Results from each validator that evaluated this
            problem
    """

    problem_id: UUID
    category: None | str | Unset = UNSET
    validator_results: list[ValidatorProblemResult] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        problem_id = str(self.problem_id)

        category: None | str | Unset
        if isinstance(self.category, Unset):
            category = UNSET
        else:
            category = self.category

        validator_results: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.validator_results, Unset):
            validator_results = []
            for validator_results_item_data in self.validator_results:
                validator_results_item = validator_results_item_data.to_dict()
                validator_results.append(validator_results_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "problem_id": problem_id,
            }
        )
        if category is not UNSET:
            field_dict["category"] = category
        if validator_results is not UNSET:
            field_dict["validator_results"] = validator_results

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.validator_problem_result import ValidatorProblemResult

        d = dict(src_dict)
        problem_id = UUID(d.pop("problem_id"))

        def _parse_category(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        category = _parse_category(d.pop("category", UNSET))

        _validator_results = d.pop("validator_results", UNSET)
        validator_results: list[ValidatorProblemResult] | Unset = UNSET
        if _validator_results is not UNSET:
            validator_results = []
            for validator_results_item_data in _validator_results:
                validator_results_item = ValidatorProblemResult.from_dict(validator_results_item_data)

                validator_results.append(validator_results_item)

        problem_progress_entry = cls(
            problem_id=problem_id,
            category=category,
            validator_results=validator_results,
        )

        problem_progress_entry.additional_properties = d
        return problem_progress_entry

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="PresignUploadResponse")


@_attrs_define
class PresignUploadResponse:
    """Response with presigned upload URL.

    Attributes:
        upload_url (str): Presigned S3 PUT URL
        results_s3_key (str): S3 key where the file will be stored
        expires_at (datetime.datetime): When the presigned URL expires
        method (str | Unset): HTTP method to use Default: 'PUT'.
    """

    upload_url: str
    results_s3_key: str
    expires_at: datetime.datetime
    method: str | Unset = "PUT"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        upload_url = self.upload_url

        results_s3_key = self.results_s3_key

        expires_at = self.expires_at.isoformat()

        method = self.method

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "upload_url": upload_url,
                "results_s3_key": results_s3_key,
                "expires_at": expires_at,
            }
        )
        if method is not UNSET:
            field_dict["method"] = method

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        upload_url = d.pop("upload_url")

        results_s3_key = d.pop("results_s3_key")

        expires_at = isoparse(d.pop("expires_at"))

        method = d.pop("method", UNSET)

        presign_upload_response = cls(
            upload_url=upload_url,
            results_s3_key=results_s3_key,
            expires_at=expires_at,
            method=method,
        )

        presign_upload_response.additional_properties = d
        return presign_upload_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, Literal, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.code_analysis_error_violations_item import CodeAnalysisErrorViolationsItem


T = TypeVar("T", bound="CodeAnalysisError")


@_attrs_define
class CodeAnalysisError:
    """422 - Static analysis found issues in submitted code.

    Attributes:
        detail (str): Error message describing what went wrong
        error_code (Literal['CODE_ANALYSIS_ERROR'] | Unset): Error code for programmatic handling Default:
            'CODE_ANALYSIS_ERROR'.
        violations (list[CodeAnalysisErrorViolationsItem] | Unset): List of violations found (rule, severity, message,
            line)
        remaining_seconds (int | None | Unset): Seconds remaining on cooldown (if cooldown was consumed by this
            submission)
    """

    detail: str
    error_code: Literal["CODE_ANALYSIS_ERROR"] | Unset = "CODE_ANALYSIS_ERROR"
    violations: list[CodeAnalysisErrorViolationsItem] | Unset = UNSET
    remaining_seconds: int | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        detail = self.detail

        error_code = self.error_code

        violations: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.violations, Unset):
            violations = []
            for violations_item_data in self.violations:
                violations_item = violations_item_data.to_dict()
                violations.append(violations_item)

        remaining_seconds: int | None | Unset
        if isinstance(self.remaining_seconds, Unset):
            remaining_seconds = UNSET
        else:
            remaining_seconds = self.remaining_seconds

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "detail": detail,
            }
        )
        if error_code is not UNSET:
            field_dict["error_code"] = error_code
        if violations is not UNSET:
            field_dict["violations"] = violations
        if remaining_seconds is not UNSET:
            field_dict["remaining_seconds"] = remaining_seconds

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.code_analysis_error_violations_item import CodeAnalysisErrorViolationsItem

        d = dict(src_dict)
        detail = d.pop("detail")

        error_code = cast(Literal["CODE_ANALYSIS_ERROR"] | Unset, d.pop("error_code", UNSET))
        if error_code != "CODE_ANALYSIS_ERROR" and not isinstance(error_code, Unset):
            raise ValueError(f"error_code must match const 'CODE_ANALYSIS_ERROR', got '{error_code}'")

        _violations = d.pop("violations", UNSET)
        violations: list[CodeAnalysisErrorViolationsItem] | Unset = UNSET
        if _violations is not UNSET:
            violations = []
            for violations_item_data in _violations:
                violations_item = CodeAnalysisErrorViolationsItem.from_dict(violations_item_data)

                violations.append(violations_item)

        def _parse_remaining_seconds(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        remaining_seconds = _parse_remaining_seconds(d.pop("remaining_seconds", UNSET))

        code_analysis_error = cls(
            detail=detail,
            error_code=error_code,
            violations=violations,
            remaining_seconds=remaining_seconds,
        )

        code_analysis_error.additional_properties = d
        return code_analysis_error

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class AgentVersionState(str, Enum):
    CANCELLED = "CANCELLED"
    DISCARDED = "DISCARDED"
    ELIGIBLE = "ELIGIBLE"
    QUEUED = "QUEUED"
    RECEIVED = "RECEIVED"
    RUNNING = "RUNNING"

    def __str__(self) -> str:
        return str(self.value)
